typedef struct {
	int id;
	int *p;
	int d;
	char *key;
} SETLIST;

static int indexFromId(SETLIST *lp, int id)
{
	int ret;
	for (ret = 0; lp[ret].id; ret++)
		if (id == lp[ret].id) return ret;
	/* fatal error */
	return -1;
}

/* Edit box list */
static SETLIST ebtbl[] = {
	{ IDC_MIXFM_EDIT, &setting.volFM, 0, "VolumeFM" },
	{ IDC_MIXSSG_EDIT, &setting.volSSG, 0, "VolumeSSG" },
	{ IDC_MIXADPCM_EDIT, &setting.volADPCM, 0, "VolumeADPCM" },
	{ IDC_MIXRHYTHM_EDIT, &setting.volRHYTHM, 0, "VolumeRHYTHM" },
	{ 0, NULL, 0, 0 },
};

/* spin control list */
static int spintbl[] =
{
	IDC_MIXFM_SPIN,
	IDC_MIXSSG_SPIN,
	IDC_MIXADPCM_SPIN,
	IDC_MIXRHYTHM_SPIN,
	0,
};

/* Check box list */
static SETLIST cbtbl[] = {
	{ 0, NULL, 0, 0 },
};

/* Channel <-> Check box table */
static struct {
	int ch;
	int id;
} cctbl[] = {
	{ -1, 0 },
};

static void S98MIXSaveSetting(void)
{
	int i;
	for (i = 0; ebtbl[i].id; i++)
	{
		SetSettingInt(ebtbl[i].key, *ebtbl[i].p);
	}
	for (i = 0; cbtbl[i].id; i++)
	{
		SetSettingInt(cbtbl[i].key, *cbtbl[i].p);
	}

}

static void S98MIXLoadSetting(void)
{
	int i;
	for (i = 0; ebtbl[i].id; i++)
	{
		*ebtbl[i].p = GetSettingInt(ebtbl[i].key, ebtbl[i].d);
	}
	for (i = 0; cbtbl[i].id; i++)
	{
		*cbtbl[i].p = GetSettingInt(cbtbl[i].key, cbtbl[i].d);
	}

	S98MIXSaveSetting();
}

static void S98MIXApplySetting(HWND hwndDlg)
{
	S98MIXSaveSetting();	/* SAVE ONLY */
}

static BOOL CALLBACK S98MIXSettingProc(
	HWND hwndDlg,
	UINT uMsg,
	WPARAM wParam,
	LPARAM lParam
)
{
	int i;
	static int active = 0;
	BOOL bMsgResult = FALSE;
	switch (uMsg)
	{
		case WM_INITDIALOG:
			active = 0;
			return TRUE;
		case WM_NOTIFY:
			switch (((LPNMHDR)lParam)->code)
			{
				case PSN_RESET:
					active = 0;
					break;
				case PSN_APPLY:
					S98MIXApplySetting(hwndDlg);
					SetWindowLong(hwndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
					return TRUE;
				case PSN_KILLACTIVE:
					active = 0;
					/* allow this page to receive PSN_APPLY */
					SetWindowLong(hwndDlg, DWL_MSGRESULT, FALSE);
					return TRUE;
				case PSN_SETACTIVE:
					for (i = 0; spintbl[i]; i++)
					{
						SendDlgItemMessage(hwndDlg, spintbl[i], UDM_SETRANGE, 0, MAKELONG(60,-20));
					}
					for (i = 0; ebtbl[i].id; i++)
					{
						SetDlgItemInt(hwndDlg,ebtbl[i].id , *ebtbl[i].p, TRUE);
					}
					for (i = 0; cbtbl[i].id; i++)
					{
						SendDlgItemMessage(hwndDlg, cbtbl[i].id, BM_SETCHECK, *cbtbl[i].p ? BST_CHECKED : BST_UNCHECKED, 0);
					}
					for (i = 0; cctbl[i].id; i++)
					{
						SendDlgItemMessage(hwndDlg, cctbl[i].id, BM_SETCHECK, setting.chenable[cctbl[i].ch] ? BST_CHECKED : BST_UNCHECKED, 0);
					}
					active = 1;
					/* accepts activation */
					SetWindowLong(hwndDlg, DWL_MSGRESULT, 0);
					return TRUE;
			}
			break;
		case WM_COMMAND:
			for (i = 0; ebtbl[i].id; i++)
			{
				if (ebtbl[i].id == LOWORD(wParam))
				{
					if (EN_CHANGE == HIWORD(wParam))
					{
						int r;
						char buf[MAX_PATH];
						r = GetDlgItemText(hwndDlg, ebtbl[i].id, buf, sizeof(buf));
						if (active && r != 0 && *buf != '\0')
						{
							*ebtbl[i].p = atoi(buf);
						}
						return TRUE;
					}
				}
			}
			for (i = 0; cbtbl[i].id; i++)
			{
				if (cbtbl[i].id == LOWORD(wParam))
				{
					if (BN_CLICKED == HIWORD(wParam))
					{
						*cbtbl[i].p = BST_CHECKED == SendDlgItemMessage(hwndDlg, cbtbl[i].id, BM_GETCHECK, 0, 0);
						return TRUE;
					}
				}
			}
			for (i = 0; cctbl[i].id; i++)
			{
				if (cctbl[i].id == LOWORD(wParam))
				{
					if (BN_CLICKED == HIWORD(wParam))
					{
						setting.chenable[cctbl[i].ch] = BST_CHECKED == SendDlgItemMessage(hwndDlg, cctbl[i].id, BM_GETCHECK, 0, 0);
						return TRUE;
					}
				}
			}
			break;
	}
	return FALSE;
}
